import json
import os
import re
from typing import Any, NotRequired, TypedDict

from rpyt.crowdin.common import CROWDIN_DIR, DialogueEntry
from rpyt.crowdin.processor import Processor


# as specified in: https://crowdin.com/store/apps/json-with-context
class Data(TypedDict):
    text: str
    crowdinContext: str


class Context(TypedDict):
    content: str
    values: NotRequired[dict[str, dict[str, Any]]]


class Contexts(TypedDict):
    standard: Context
    dialogue: Context


class JsonWithContextProcessor(Processor):
    CONTEXTS = f"{CROWDIN_DIR}/contexts.json"

    def __init__(self, contexts: str | None = None):
        if contexts is None and os.path.exists(self.CONTEXTS):
            contexts = self.CONTEXTS
        if contexts:
            with open(contexts, encoding="utf-8") as fo:
                self.contexts = Contexts(**json.load(fo))
        else:
            self.contexts = None

    def dump(self, dialogue_entry: DialogueEntry):
        ctx: Context | None = (
            (
                self.contexts["dialogue"]
                if dialogue_entry.identifier
                else self.contexts["standard"]
            )
            if self.contexts
            else None
        )
        if not ctx:
            return dialogue_entry.dialogue
        return Data(
            text=dialogue_entry.dialogue,
            crowdinContext=ctx["content"].format(
                **self.get_values(dialogue_entry, ctx.get("values", {}))
            ),
        )

    def load(self, value: Data):
        return value["text"]

    def get_values(
        self, dialogue_entry: DialogueEntry, values: dict[str, dict[str, Any]]
    ):
        de = dialogue_entry._asdict()
        for k, v in de.items():
            if k in values:
                key = next(
                    filter(lambda e: re.search(e, str(v)), values[k]), None
                )
                if key:
                    aval = values[k][key]
                    de[k] = (
                        str.format(aval, **de)
                        if isinstance(aval, str)
                        else aval
                    )
        return de



from fire import Fire

from rpyt.crowdin import (
    create_inputs,
    read_common,
    read_dialogue,
    translate,
    translate_outputs,
)


class RpytCrowdin:
    read_common = staticmethod(read_common)
    read_dialogue = staticmethod(read_dialogue)
    create_inputs = staticmethod(create_inputs)
    translate = staticmethod(translate)
    translate_outputs = staticmethod(translate_outputs)


class Rpyt:
    crowdin = RpytCrowdin


def main():
    Fire(Rpyt)


if __name__ == "__main__":
    main()



import json
import os
import re
import shutil
from importlib.resources import files
from typing import Any, cast

from rpyt.common import hash_object, load_from_module, read_tab, rreplace
from rpyt.crowdin.common import (
    CROWDIN_INPUTS_DIR,
    CROWDIN_OUTPUTS_DIR,
    GAME_COMMON,
    GAME_DIALOGUE,
    GAME_DIR,
    GAME_TL_DIR,
    RENPY_COMMON_DIR,
    RENPY_COMMON_NAME,
    RENPY_RPY_EXTENSION,
    RENPY_RPYM_EXTENSION,
    DialogueEntry,
)
from rpyt.crowdin.processor import Processor

RESOURCES_COMMON_RPYM = str(
    files("rpyt.resources").joinpath(
        f"{RENPY_COMMON_NAME}.{RENPY_RPYM_EXTENSION}.txt"
    )
).replace("\\\\", "/")
CROWDIN_DEFAULT_PROCESSOR: type[Processor] | str = (
    "rpyt.crowdin.json_with_context_processor.JsonWithContextProcessor"
)


def read_common(file=GAME_COMMON):
    if not os.path.exists(file):
        file = RESOURCES_COMMON_RPYM
    with open(file, encoding="utf-8") as fo:
        common = fo.read()
    pattern = r"^.*#\s(.+):(\d+)\n.*old\s['\"](.*)['\"]$"
    matches = re.findall(pattern, common, re.MULTILINE)
    rv: list[DialogueEntry] = []
    for match in matches:
        tpl = (None, None, match[2], match[0], int(match[1]), None)
        rv.append(DialogueEntry(hash_object(tpl), *tpl))
    return rv


def read_dialogue(file=GAME_DIALOGUE):
    rows = read_tab(file, "header")
    rows.pop(0)
    dialogue = list(
        map(
            lambda e: DialogueEntry(hash_object(e), *e),
            rows,
        )
    )
    return dialogue


def create_inputs(
    dir=CROWDIN_INPUTS_DIR,
    dialogue=GAME_DIALOGUE,
    common=GAME_COMMON,
    processor=CROWDIN_DEFAULT_PROCESSOR,
    **kwargs: Any,
):
    cdialogue = read_common(common)
    mdialogue = read_dialogue(dialogue)
    adialouge = cdialogue + mdialogue

    if os.path.exists(dir):
        shutil.rmtree(dir)
    os.makedirs(dir, exist_ok=True)

    files: dict[str, dict[str, Any]] = {}

    if isinstance(processor, str):
        processor = cast(type[Processor], load_from_module(processor))
    iprocessor = processor(**kwargs)

    for de in adialouge:
        path = de.filename
        if path.startswith(RENPY_COMMON_DIR):
            path = f"{dir}/{RENPY_COMMON_NAME}.{RENPY_RPY_EXTENSION}.json"
        else:
            path = rreplace(
                path.removeprefix(GAME_DIR + "/"),
                RENPY_RPYM_EXTENSION,
                RENPY_RPY_EXTENSION,
                1,
            )
            path = f"{dir}/{path}.json"
        if path not in files:
            file = files[path] = {}
        else:
            file = files[path]
        file[de.hash] = iprocessor.dump(de)

    totstrs = 0

    for path, content in files.items():
        dirname = os.path.dirname(path)
        os.makedirs(dirname, exist_ok=True)
        with open(path, "w", encoding="utf-8") as fo:
            json.dump(content, fo, indent=4)
            totstrs += len(content)
            print(f"dumped {len(content)} string(s) into '{path}'")

    print(f"created {len(files)} file(s); dumped {totstrs} string(s)")


def translate(
    file: str,
    language: str,
    indent=4,
    dialogue: str | list[DialogueEntry] = GAME_DIALOGUE,
    common: str | list[DialogueEntry] = GAME_COMMON,
    processor=CROWDIN_DEFAULT_PROCESSOR,
    **kwargs: Any,
):
    cdialogue = read_common(common) if isinstance(common, str) else common
    mdialogue = (
        read_dialogue(dialogue) if isinstance(dialogue, str) else dialogue
    )
    adialouge = cdialogue + mdialogue

    with open(file, "r", encoding="utf-8") as fo:
        data: dict[str, Any] = json.load(fo)

    dstring_temp = """# {file}:{line}
translate {lang} {ident}:

{indent}# {oscript}
{indent}{nscript}
"""
    nstring_temp = """{indent}# {file}:{line}
{indent}old "{old}"
{indent}new "{new}"
"""

    strings: list[str] = []
    ndes: list[tuple[Any, DialogueEntry]] = []

    if isinstance(processor, str):
        processor = cast(type[Processor], load_from_module(processor))
    iprocessor = processor(**kwargs)

    for hkey, value in data.items():
        de = next((de for de in adialouge if de.hash == hkey), None)
        if de:
            if de.identifier and de.renpy_script:
                strings.append(
                    dstring_temp.format(
                        file=de.filename,
                        line=de.line_number,
                        lang=language,
                        ident=de.identifier,
                        oscript=rreplace(
                            de.renpy_script, "[what]", de.dialogue, 1
                        ),
                        nscript=rreplace(
                            de.renpy_script,
                            "[what]",
                            iprocessor.load(value),
                            1,
                        ),
                        indent=" " * indent,
                    )
                )
            else:
                ndes.append((value, de))

    if ndes:
        strings.append(f"translate {language} strings:\n")

    for value, de in ndes:
        if (not de.identifier) and (not de.renpy_script):
            strings.append(
                nstring_temp.format(
                    file=de.filename,
                    line=de.line_number,
                    old=de.dialogue,
                    new=iprocessor.load(value),
                    indent=" " * indent,
                )
            )

    return "\n".join(strings)


def translate_outputs(
    dir=CROWDIN_OUTPUTS_DIR,
    languages: dict[str, str] | list[str] | None = None,
    tl=GAME_TL_DIR,
    inputs=CROWDIN_INPUTS_DIR,
    dialogue=GAME_DIALOGUE,
    common=GAME_COMMON,
    processor=CROWDIN_DEFAULT_PROCESSOR,
    **kwargs: Any,
):
    cdialogue = read_common(common)
    mdialogue = read_dialogue(dialogue)

    alangs = list(
        map(lambda e: e.name, (e for e in os.scandir(dir) if e.is_dir()))
    )
    lmap: dict[str, str] = {}
    if languages is None:
        lmap.update({lang: lang for lang in alangs})
    elif isinstance(languages, list):
        lmap.update({lang: lang for lang in languages})
    else:
        lmap.update(languages)

    files: dict[str, str] = {}

    for inlang, outlang in lmap.items():
        inlangdir = f"{dir}/{inlang}"
        outlangdir = f"{tl}/{outlang}"

        nested_inlangdir = f"{inlangdir}/{inputs}"

        for dirpath, _, filenames in os.walk(inlangdir):
            dirpath = dirpath.replace("\\", "/")
            if dirpath.startswith(nested_inlangdir):
                for filename in filenames:
                    if filename.endswith(
                        (
                            f"{RENPY_RPY_EXTENSION}.json",
                            f"{RENPY_RPYM_EXTENSION}.json",
                        )
                    ):
                        file = f"{dirpath}/{filename}"
                        outpath = f"{outlangdir}/{dirpath.removeprefix(nested_inlangdir)[1:]}/{filename}".replace(
                            "//", "/"
                        ).removesuffix(".json")
                        files[outpath] = translate(
                            file,
                            outlang,
                            dialogue=mdialogue,
                            common=cdialogue,
                            processor=processor,
                            **kwargs,
                        )

    for path, content in files.items():
        dirname = os.path.dirname(path)
        os.makedirs(dirname, exist_ok=True)
        with open(path, "w", encoding="utf-8") as fo:
            fo.write(content)
            print(f"created '{path}'")

    print(f"created {len(files)} file(s)")



from abc import ABC, abstractmethod
from typing import Any

from rpyt.crowdin.common import DialogueEntry


class Processor(ABC):
    def __init__(self, **kwargs: Any):
        pass

    @abstractmethod
    def dump(self, dialogue_entry: DialogueEntry):
        pass

    @abstractmethod
    def load(self, value: Any) -> str:
        pass



from typing import NamedTuple


class DialogueEntry(NamedTuple):
    hash: str
    identifier: str | None
    character: str | None
    dialogue: str
    filename: str
    line_number: int
    renpy_script: str | None


RENPY_COMMON_NAME = "common"
RENPY_RPY_EXTENSION = "rpy"
RENPY_RPYM_EXTENSION = f"{RENPY_RPY_EXTENSION}m"
RENPY_COMMON_DIR = f"renpy/{RENPY_COMMON_NAME}"
GAME_DIR = "game"
GAME_TL_DIR = f"{GAME_DIR}/tl"
GAME_COMMON = f"{GAME_TL_DIR}/None/{RENPY_COMMON_NAME}.{RENPY_RPYM_EXTENSION}"
GAME_DIALOGUE = "dialogue.tab"
CROWDIN_DIR = "crowdin"
CROWDIN_INPUTS_DIR = f"{CROWDIN_DIR}/inputs"
CROWDIN_OUTPUTS_DIR = f"{CROWDIN_DIR}/outputs"



from rpyt.crowdin.common import DialogueEntry
from rpyt.crowdin.processor import Processor


class StringProcessor(Processor):
    def dump(self, dialogue_entry: DialogueEntry):
        return dialogue_entry.dialogue

    def load(self, value: str):
        return value
